#include "Matrix.h"
#include <math.h>
#include <stdio.h>
#include <Windows.h>

Matrix::Matrix(int _m, int _n) : m(_m), n(_n) {
	int i;

	data = (double*)malloc(m*n * sizeof(double));
	table = (double**)malloc(m * sizeof(double*));

	memset(data, 0, m*n * sizeof(double));

	for (i = 0; i < m; i++) table[i] = &data[i*n];
}

void Matrix::Multiply(Matrix* a, Matrix* b, Matrix* c) {
	int i, j;
	int l;

	for (i = 0; i < c->m; i++) {
		for (j = 0; j < c->n; j++) {
			c->table[i][j] = 0;
			for (l = 0; l < a->n; l++) {
				c->table[i][j] += a->table[i][l] * b->table[l][j];
			}
		}
	}
}

void Matrix::Transpose(Matrix* a, Matrix* b) {
	int i, j;

	for (i = 0; i < b->m; i++) {
		for (j = 0; j < b->n; j++) {
			b->table[i][j] = a->table[j][i];
		}
	}
}

void Matrix::Invert() {
	int s;
	double pv, pav, temp, tt;
	int i, ik, j, jk, k;
	double det;

	s = n * sizeof(double);
	double* pc = (double*)malloc(s);
	double* pl = (double*)malloc(s);
	double* cs = (double*)malloc(s);

	det = 1;
	for (i = 0; i < n; i++) {
		pc[i] = 0;
		pl[i] = 0;
		cs[i] = 0;
	}

	for (k = 0; k < n; k++) {
		pv = table[k][k];
		ik = k;
		jk = k;
		pav = fabs(pv);
		for (i = k; i < n; i++) {
			for (j = k; j < n; j++) {
				temp = fabs(table[i][j]);
				if (temp > pav) {
					pv = table[i][j];
					pav = fabs(pv);
					ik = i;
					jk = j;
				}
			}
		}

		pc[k] = jk;
		pl[k] = ik;

		if (ik != k) det = -det;
		if (jk != k) det = -det;
		det = det * pv;
		temp = fabs(det);
		if (temp < 1e-50) throw("Determinant is zero");

		if (ik != k) {
			for (i = 0; i < n; i++) {
				tt = table[ik][i];
				table[ik][i] = table[k][i];
				table[k][i] = tt;
			}
		}

		if (jk != k) {
			for (i = 0; i < n; i++) {
				tt = table[i][jk];
				table[i][jk] = table[i][k];
				table[i][k] = tt;
			}
		}

		for (i = 0; i < n; i++) {
			cs[i] = table[i][k];
			table[i][k] = 0;
		}

		cs[k] = 0;
		table[k][k] = 1;

		temp = fabs(pv);

		for (i = 0; i < n; i++) {
			table[k][i] = table[k][i] / pv;
		}

		for (j = 0; j < n; j++) {
			if (j == k) j++;
			if (j < n) {
				for (i = 0; i < n; i++) {
					table[j][i] = table[j][i] - cs[j] * table[k][i];
				}
			}
		}
	}

	for (i = n - 1; i >= 0; i--) {
		ik = (int)pc[i];
		if (ik != i) {
			for (j = 0; j < n; j++) {
				tt = table[i][j];
				table[i][j] = table[ik][j];
				table[ik][j] = tt;
			}
		}
	}

	for (j = n - 1; j >= 0; j--) {
		jk = (int)pl[j];
		if (jk != j) {
			for (i = 0; i < n; i++) {
				tt = table[i][j];
				table[i][j] = table[i][jk];
				table[i][jk] = tt;
			}
		}
	}

	free(pc);
	free(pl);
	free(cs);
}

void Matrix::Row_8(int row, double a, double b, double c, double d, double e, double f, double g, double h) {
	table[row][0] = a;
	table[row][1] = b;
	table[row][2] = c;
	table[row][3] = d;
	table[row][4] = e;
	table[row][5] = f;
	table[row][6] = g;
	table[row][7] = h;
}

void Matrix::Debug() {
	Debug((char*)"Matrix");
}

void Matrix::Debug(char* name) {
	int x, y;
	char line[2048];
	char next[128];

	sprintf_s(line, "%s {", name);
	OutputDebugString(line);

	for (y = 0; y < m; y++) {
		sprintf_s(line, "\t");
		for (x = 0; x < n; x++) {
			sprintf_s(next, "%8.6f\t", table[y][x]);
			strcat_s(line, next);
		}
		OutputDebugString(line);
	}
	OutputDebugString("}");
}
